package Perinci::Examples::Version;

our $DATE = '2016-01-22'; # DATE
our $VERSION = '0.66'; # VERSION

our %SPEC;

$SPEC{':package'} = {
    v => 1.1,
    summary => 'Package that contains entity_v in its Rinci package metadata',
    entity_v => 9.99,
};

# ABSTRACT: Package that contains entity_v in its Rinci package metadata

__END__

=pod

=encoding UTF-8

=head1 NAME

Perinci::Examples::Version - Package that contains entity_v in its Rinci package metadata

=head1 VERSION

This document describes version 0.66 of Perinci::Examples::Version (from Perl distribution Perinci-Examples), released on 2016-01-22.

=head1 DESCRIPTION

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Examples>.

=head1 SOURCE

Source repository is at L<https://github.com/sharyanto/perl-Perinci-Examples>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Perinci-Examples>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2016 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
